@@ -3,7 +3,9 @@ package App::Dest;
 
 use strict;
 use warnings;
+use 5.016;
 
+use Cwd 'getcwd';
 use File::Basename qw( dirname basename );
 use File::Copy::Recursive 'dircopy';
 use File::DirCompare ();
@@ -13,8 +15,9 @@ use IPC::Run 'run';
 use Text::Diff ();
 use Try::Tiny qw( try catch );
 
+our $VERSION = '1.14'; # VERSION
 
-our $VERSION = '1.13'; # VERSION
+my %seen_files;
 
 sub init {
     my ($self) = @_;
@@ -52,9 +55,10 @@ sub init {
 
 sub add {
     my ( $self, $dir ) = @_;
+    $dir //= '';
     $dir =~ s|/$||;
 
-    die "Not in project root directory or project not initialized\n" unless ( -d '.dest' );
+    _find_root_dir();
     die "No directory specified; usage: dest add [directory]\n" unless ($dir);
     die "Directory specified does not exist\n" unless ( -d $dir );
     die "Directory $dir already added\n" if ( grep { $dir eq $_ } $self->_watches );
@@ -68,10 +72,11 @@ sub add {
 
 sub rm {
     my ( $self, $dir ) = @_;
+    $dir //= '';
     $dir =~ s|/$||;
 
     die "Not in project root directory or project not initialized\n" unless ( -d '.dest' );
-    die "No directory specified; usage: dest add [directory]\n" unless ($dir);
+    die "No directory specified; usage: dest rm [directory]\n" unless ($dir);
     die "Directory $dir not currently tracked\n" unless ( grep { $dir eq $_ } $self->_watches );
 
     my @watches = $self->_watches;
@@ -83,13 +88,16 @@ sub rm {
 }
 
 sub make {
-    my ( $self, $path ) = @_;
+    my ( $self, $path, $ext ) = @_;
     die "No name specified; usage: dest make [path]\n" unless ($path);
 
+    $ext = '.' . $ext if ( defined $ext );
+    $ext //= '';
+
     eval {
         mkpath($path);
         for ( qw( deploy verify revert ) ) {
-            open( my $file, '>', "$path/$_" ) or die;
+            open( my $file, '>', "$path/$_$ext" ) or die;
             print $file "\n";
         }
     };
@@ -103,7 +111,7 @@ sub list {
     my ( $self, $path ) = @_;
 
     if ($path) {
-        print join( ' ', map { "$path/$_" } qw( deploy verify revert ) ), "\n";
+        print join( ' ', map { <"$path/$_*"> } qw( deploy verify revert ) ), "\n";
     }
     else {
         for my $path ( $self->_watches ) {
@@ -113,8 +121,8 @@ sub list {
                 follow   => 1,
                 no_chdir => 1,
                 wanted   => sub {
-                    return unless ( m|/deploy$| );
-                    ( my $action = $_ ) =~ s|/deploy$||;
+                    return unless ( m|/deploy(?:\.[^\/]+)?| );
+                    ( my $action = $_ ) =~ s|/deploy(?:\.[^\/]+)?||;
                     print '  ', $action, "\n";
                 },
             }, $path );
@@ -126,8 +134,7 @@ sub list {
 
 sub status {
     my ($self) = @_;
-
-    die "Not in project root directory or project not initialized\n" unless ( -d '.dest' );
+    _find_root_dir();
 
     if ( -f 'dest.watch' ) {
         my $diff = Text::Diff::diff( '.dest/watch', 'dest.watch' );
@@ -191,9 +198,8 @@ sub diff {
 }
 
 sub update {
-    my $self  = shift;
-
-    die "Not in project root directory or project not initialized\n" unless ( -d '.dest' );
+    my $self = shift;
+    _find_root_dir();
 
     if ( -f 'dest.watch' ) {
         my @watches = $self->_watches;
@@ -229,7 +235,6 @@ sub update {
             $self->deploy($b);
         }
         else {
-
             $a =~ s|\.dest/||;
             $a =~ s|/(\w+)$||;
             $b =~ s|/(\w+)$||;
@@ -251,26 +256,25 @@ sub update {
 
 sub verify {
     my ( $self, $path ) = @_;
-    die "Not in project root directory or project not initialized\n" unless ( -d '.dest' );
+    _find_root_dir();
     return $self->_action( $path, 'verify' );
 }
 
 sub deploy {
     my ( $self, $name, $redeploy ) = @_;
     die "File to deploy required; usage: dest deploy file\n" unless ($name);
-    die "Not in project root directory or project not initialized\n" unless ( -d '.dest' );
+    _find_root_dir();
     my $rv = $self->_action( $name, 'deploy', $redeploy );
-    $self->verify($name);
-    dircopy( $name, ".dest/$name" );
+    dircopy( $_, ".dest/$_" ) for ( grep { s|/deploy[^/]*$|| } keys %seen_files );
     return $rv;
 }
 
 sub revert {
     my ( $self, $name ) = @_;
     die "File to revert required; usage: dest revert file\n" unless ($name);
-    die "Not in project root directory or project not initialized\n" unless ( -d '.dest' );
+    _find_root_dir();
     my $rv = $self->_action( ".dest/$name", 'revert' );
-    rmtree(".dest/$name");
+    rmtree(".dest/$_") for ( map { s|^.dest/||; $_ } grep { s|/revert[^/]*$|| } keys %seen_files );
     return $rv;
 }
 
@@ -287,7 +291,7 @@ sub revdeploy {
 
 sub clean {
     my ($self) = @_;
-    die "Not in project root directory or project not initialized\n" unless ( -d '.dest' );
+    _find_root_dir();
     for ( $self->_watches ) {
         rmtree(".dest/$_");
         dircopy( $_, ".dest/$_" );
@@ -297,7 +301,7 @@ sub clean {
 
 sub preinstall {
     my ($self) = @_;
-    die "Not in project root directory or project not initialized\n" unless ( -d '.dest' );
+    _find_root_dir();
     for ( $self->_watches ) {
         rmtree(".dest/$_");
         mkdir(".dest/$_");
@@ -311,6 +315,14 @@ sub watches {
     return 0;
 }
 
+sub _find_root_dir {
+    while ( getcwd() ne '/' ) {
+        return if ( -d '.dest' );
+        chdir('..');
+    }
+    die "Failed to find project root dir; run init in project root dir to resolve\n";
+}
+
 sub _watches {
     open( my $watch, '<', '.dest/watch' ) or die "Unable to read .dest/watch file\n";
     return sort { $a cmp $b } map { chomp; $_ } <$watch>;
@@ -318,20 +330,24 @@ sub _watches {
 
 sub _action {
     my ( $self, $path, $type, $redeploy ) = @_;
+    %seen_files = ();
 
     if ($path) {
-        unless ( -f "$path/$type" ) {
-            my $this_file = substr( $path, 7 );
+        my @files = <"$path/$type*">;
+        my $file  = $files[0];
+
+        unless ($file) {
+            my $this_file = ( split( '/', $path ) )[-1];
             die "Unable to $type $this_file (perhaps action has already occured)\n";
         }
-        $self->_execute( "$path/$type", $redeploy ) or die "Failed to $type $path\n";
+        $self->_execute( $file, $redeploy ) or die "Failed to $type $path\n";
     }
     else {
         find( {
             follow   => 1,
             no_chdir => 1,
             wanted   => sub {
-                return unless ( /\/$type$/ );
+                return unless ( /\/$type/ );
                 $self->_execute($_) or die "Failed to $type $_\n";
             },
         }, $self->_watches );
@@ -340,70 +356,93 @@ sub _action {
     return 0;
 }
 
-{
-    my %seen_files;
-    sub _execute {
-        my ( $self, $file, $run_quiet, $is_dependency ) = @_;
-        return if ( $seen_files{$file}++ );
+sub _execute {
+    my ( $self, $file, $run_quiet, $is_dependency ) = @_;
+    return if ( $seen_files{$file}++ );
 
-        my @nodes = split( '/', $file );
-        my $type = pop @nodes;
-        ( my $action = join( '/', @nodes ) ) =~ s|^\.dest/||;
+    my @nodes = split( '/', $file );
+    my $type = pop @nodes;
+    ( my $action = join( '/', @nodes ) ) =~ s|^\.dest/||;
 
-        if (
-            ( $type eq 'deploy' and not $run_quiet and -f '.dest/' . $file ) or
-            ( $type eq 'revert' and not -f $file )
-        ) {
-            if ( $is_dependency ) {
-                return;
-            }
-            else {
-                die 'Action already '. $type . "ed\n";
-            }
+    $type =~ s/\..*$//;
+
+    if (
+        ( $type eq 'deploy' and not $run_quiet and -f '.dest/' . $file ) or
+        ( $type eq 'revert' and not -f $file )
+    ) {
+        if ( $is_dependency ) {
+            return;
         }
+        else {
+            die 'Action already '. $type . "ed\n";
+        }
+    }
 
-        open( my $content, '<', $file ) or die "Unable to read $file\n";
+    open( my $content, '<', $file ) or die "Unable to read $file\n";
 
-        $self->_execute( "$_/$type", undef, 'dependency' ) for (
-            grep { defined }
-            map { /dest\.prereq\b[\s:=-]+(.+?)\s*$/; $1 || undef }
-            grep { /dest\.prereq/ } <$content>
+    for (
+        grep { defined }
+        map { /dest\.prereq\b[\s:=-]+(.+?)\s*$/; $1 || undef }
+        grep { /dest\.prereq/ } <$content>
+    ) {
+        my @files = <"$_/$type*">;
+        die "Unable to find prereq \"$_/$type*\"\n" unless ( $files[0] );
+        $self->_execute( $files[0], $run_quiet, 'dependency' ) or return 0 if (
+            ( $type eq 'deploy' and not -f '.dest/' . $files[0] ) or
+            ( $type eq 'revert' and -f '.dest/' . $files[0] )
         );
+    }
 
-        my $wrap;
-        shift @nodes if ( $nodes[0] eq '.dest' );
-        while (@nodes) {
-            my $path = join( '/', @nodes );
-            if ( -f "$path/dest.wrap" ) {
-                $wrap = "$path/dest.wrap";
-                last;
-            }
-            pop @nodes;
+    my $wrap;
+    shift @nodes if ( $nodes[0] eq '.dest' );
+    while (@nodes) {
+        my $path = join( '/', @nodes );
+        if ( -f "$path/dest.wrap" ) {
+            $wrap = "$path/dest.wrap";
+            last;
         }
+        pop @nodes;
+    }
 
-        if ( $type eq 'verify' ) {
-            my ( $out, $err );
-
+    my ( $out, $err );
+    my $run = sub {
+        eval {
             run(
                 [ grep { defined } ( ($wrap) ? $wrap : undef ), $file ],
                 \undef, \$out, \$err,
-            ) or die "Failed to execute $file\n";
-
-            chomp($out);
-            return ($err) ? 0 : $out if ($run_quiet);
-
-            die "$err\n" if ($err);
-            print '', ( ($out) ? 'ok' : 'not ok' ) . " - verify: $action\n";
-        }
-        else {
-            print "begin - $type: $action\n";
-            run( [ grep { defined } ( ($wrap) ? $wrap : undef ), $file ] ) or die "Failed to execute $file\n";
-            $file =~ s|^\.dest/||;
-            print "ok - $type: $action\n";
+            );
+        };
+        if ( $@ or $err ) {
+            ( my $err_str = $@ || $err ) =~ s/\s*at\s+.*$//;
+            chomp($err_str);
+            die "Failed to execute $file: $err_str\n";
         }
+    };
 
-        return 1;
+    if ( $type eq 'verify' ) {
+        $run->();
+
+        chomp($out);
+        return ($err) ? 0 : $out if ($run_quiet);
+
+        die "$err\n" if ($err);
+        print '', ( ($out) ? 'ok' : 'not ok' ) . " - verify: $action\n";
+        return 0 if ( not $out );
     }
+    else {
+        print "begin - $type: $action\n";
+        $run->();
+
+        $file =~ s|^\.dest/||;
+        print "ok - $type: $action\n";
+
+        if ( $type eq 'deploy' ) {
+            ( my $verify_file = $file ) =~ s|([^/]+)$| 'verify' . substr( $1, 6 ) |e;
+            return $self->_execute($verify_file);
+        }
+    }
+
+    return 1;
 }
 
 1;
@@ -420,7 +459,7 @@ App::Dest - Deployment State Manager
 
 =head1 VERSION
 
-version 1.13
+version 1.14
 
 =for markdown [![Build Status](https://travis-ci.org/gryphonshafer/dest.svg)](https://travis-ci.org/gryphonshafer/dest)
 [![Coverage Status](https://coveralls.io/repos/gryphonshafer/dest/badge.png)](https://coveralls.io/r/gryphonshafer/dest)
@@ -432,7 +471,7 @@ dest COMMAND [DIR || NAME]
     dest init            # initialize dest for a project
     dest add DIR         # add a directory to dest tracking list
     dest rm DIR          # remove a directory from dest tracking list
-    dest make NAME       # create a named template set (set of 3 files)
+    dest make NAME [EXT] # create a named template set (set of 3 files)
     dest watches         # returns a list of watched directories
     dest list [NAME]     # dump a list of the template set (set of 3 files)
     dest status          # check status of tracked directories
@@ -513,7 +552,7 @@ the verify file let's you verify the deploy file worked.
 
 This removes a directory from the dest tracking list.
 
-=head2 make NAME
+=head2 make NAME [EXT]
 
 This is a helper command. Given a directory you've already added, it will create
 the subdirectory and deploy, revert, and verify files.
@@ -526,6 +565,14 @@ So if you want, you can do something like this:
 
     vi `dest make db/schema`
 
+Optionally, you can specify an extention for the created files. For example:
+
+    vi `dest make db/schema sql`
+    # this will create and open in vi:
+    #    db/schema/deploy.sql
+    #    db/schema/revert.sql
+    #    db/schema/verify.sql
+
 =head2 watches
 
 Returns a list of tracked or watched directories.
